import { WebGPUObjectError } from '../utils/errors.js'

export class BindGroup {
  _device
  _handle

  get handle() {
    return this._handle
  }

  /**
   * @param {GPUDevice} device
   * @param {GPUBindGroup} bindGroup
   */
  constructor(device, bindGroup) {
    this._device = device
    this._handle = bindGroup
  }

  /**
   * @param {GPUDevice} device
   * @param {GPUBindGroupDescriptor} descriptor
   */
  static create(device, descriptor) {
    try {
      return new BindGroup(
        device,
        device.createBindGroup(descriptor)
      )
    } catch (err) {
      throw WebGPUObjectError.from(err, BindGroup)
    }
  }

  destroy() {
    this._handle = null
  }
}

