import { WebGPUObjectError } from '../utils/errors.js'
import { ResourceType } from '../utils/internal-enums.js'

/** @import { BindGroupEntry } from '../core/graphics-device.js' */

export class Texture {
  _device
  _handle

  /** @type {GPUTextureView | undefined} */
  _defaultView

  get handle() {
    return this._handle
  }

  get width() {
    return this._handle.width
  }

  get height() {
    return this._handle.height
  }

  get format() {
    return this._handle.format
  }

  get depthOrArrayLayers() {
    return this._handle.depthOrArrayLayers
  }

  get usage() {
    return this._handle.usage
  }

  get dimension() {
    return this._handle.dimension
  }

  get mipLevelCount() {
    return this._handle.mipLevelCount
  }

  get label() {
    return this._handle.label
  }

  get resourceType() {
    return ResourceType.TextureView
  }

  /**
   * @param {GPUDevice} device
   * @param {GPUTexture} texture
   */
  constructor(device, texture) {
    this._device = device
    this._handle = texture
  }

  /**
   * @param {GPUDevice} device
   * @param {GPUTextureDescriptor} descriptor
   */
  static create(device, descriptor) {
    try {
      return new Texture(
        device,
        device.createTexture(descriptor)
      )
    } catch (err) {
      throw WebGPUObjectError.from(err, Texture)
    }
  }

  /**
   * @param {GPUTextureViewDescriptor} [descriptor]
   * @throws {TextureError}
   */
  createDefaultView(descriptor) {
    if (!descriptor && this._defaultView) {
      return this._defaultView
    }

    try {
      const view = this._handle.createView(descriptor)

      if (!descriptor) {
        this._defaultView = view
      }

      return view
    } catch (err) {
      throw WebGPUObjectError.from(err, Texture)
    }
  }

  getView() {
    return this.createDefaultView()
  }

  destroy() {
    this._handle?.destroy()
    this._handle = undefined
    this._defaultView = undefined
  }
}

